function extend(t, mt)
	local __mt = getmetatable(t) or {}
	for k, v in pairs(mt) do
		__mt[k] = v
	end
	return setmetatable(t, __mt)
end

function constant(t)
	return extend(t, {
		__newindex = function(t, n, v)
			error("Attempt to change a constant", 0)
		end
	})
end

function protected(t)
	local __t = setmetatable({}, {
		__newindex = function(t, k, v)
			if type(v) == "function" then
				rawset(t, k, function(...)
					local status, message = pcall(v, ...)
					if status then
						return message
					else
						print(message)
					end
				end)
			else
				rawset(t, k, v)
			end
		end
	})
	for k, v in pairs(t) do
		__t[k] = v
	end
	return __t
end

function eval(script, env, eh)
	if type(script) == "function" then
		setfenv(script, env)
		return xpcall(script, eh)
	else
		local fn, err = loadstring(script)
		if fn then
			setfenv(fn, env)
			return xpcall(fn, eh)
		else
			return nil, err
		end
	end
end

function serialize(var)
	local t = type(var)
	if t == "boolean" then
		return var and "true" or "false"
	elseif t == "number" then
		return tostring(var)
	elseif t == "string" then
		return string.format("%q", var)
	elseif t == "table" then
		local s = "{"
		for k, v in pairs(var) do
			local t = type(k)
			if t == "number" then
				s = s .. serialize(v) .. ","
			elseif t == "string" or t == "boolean" or t == "table" then
				s = string.format("%s[%s]=%s,", s, serialize(k), serialize(v))
			end
		end
		return s .. "}"
	else
		return "{}"
	end
end

function deserialize(s)
	local success, result = eval("return " .. s, {})
	return success and result or {}
end

function string.lines(s)
	local lines = {}
	for line in string.gmatch(s, "([^\n]*)\n") do
		table.insert(lines, line)
	end
	table.insert(lines, string.match(s, "\n([^\n]*)$"))
	return lines
end

function string.trim(s)
	return string.match(s, "^%s*(.-)%s*$")
end
